package hostedview;

import javax.inject.Inject;

import org.eclipse.e4.core.contexts.ContextInjectionFactory;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MBasicFactory;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MPartStack;
import org.eclipse.e4.ui.model.application.ui.basic.MWindow;
import org.eclipse.e4.ui.workbench.IPresentationEngine;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchWindow;

public class LocalViewPage extends WizardPage {
	private String viewId;
	private MWindow hostWin;
	
	// Ouch !! self-injection !!
	@Inject private EPartService ps;
	@Inject private IPresentationEngine renderer;
	
	@SuppressWarnings("restriction")
	protected LocalViewPage(String pageName, String viewId) {
		super(pageName);
		this.viewId = viewId;
		
		setTitle("Eclipse 4 View Hosting");
		setMessage("This really *is* the Package Explorer");

		// This is not really necessary but it demonstrates how DI works
		// once the code below runs the '@Inject' fields defined above will
		// contain their correct values.
		IWorkbenchWindow wbw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		hostWin = (MWindow) wbw.getService(MWindow.class);
		ContextInjectionFactory.inject(this, hostWin.getContext());
	}

	@SuppressWarnings("restriction")
	@Override
	public void createControl(Composite parent) {
		Composite comp = new Composite(parent, SWT.NONE);
		comp.setLayout(new FillLayout());
		setControl(comp);
		
		// Create the model
		MUIElement hostModel = createModel();
		
		// Render the model...by 4.2 M4 we expect that this will be
		// available as a method in the EModelService
		renderModel(comp, hostWin, hostModel);
	}
	
	@SuppressWarnings("restriction")
	private MUIElement createModel() {
		// Create the view's model
		MPart viewModel = ps.createPart(viewId);
		viewModel.setCloseable(false);
		
		// Create a second 'simple' e4-based part
		MPart simple = MBasicFactory.INSTANCE.createPart();
		simple.setLabel("Simple Part");
		simple.setContributionURI("platform:/plugin/HostedView/hostedview.AReallySimplePart");
		simple.setIconURI(viewModel.getIconURI());
				
		// Wrap the parts in a stack
		MPartStack stack = MBasicFactory.INSTANCE.createPartStack();
		stack.getChildren().add(viewModel);
		stack.getChildren().add(simple);
		
		return stack;
	}
	
	@SuppressWarnings("restriction")
	private void renderModel(Composite parent, final MWindow hostWin, final MUIElement hostModel) {
		// This is subtle; unless the element is hooked into the model it won't fire events
		hostWin.getSharedElements().add(hostModel);
		
		// Render it
		renderer.createGui(hostModel, parent, hostWin.getContext());		
		
		// Clean up the shared elements list once we're done
		parent.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				hostWin.getSharedElements().remove(hostModel);
			}
		});		
	}
} 
